/************************************************************************
	main.c

    MicroSimon (MB Electronics Simon Clone)
    Copyright (C) 2011 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef MAIN_C
#define MAIN_C

// Global includes
#include <htc.h>

// Local includes
#include "hardware.h"
#include "sound.h"
#include "charlieplex.h"
#include "buttons.h"
#include "sleep.h"
#include "simon.h"

// Fuse configuration for the PIC12F683
__CONFIG(FCMDIS & IESODIS & BORDIS & UNPROTECT & MCLREN &
	PWRTDIS & WDTDIS & INTIO);	

// Main procedure
void main(void)
{
	// Set up the PIC12F683 IO pins
	GPIO   = 0b00000000;	// Set all pins to zero
	TRISIO = 0b00000001;	// Set all IO pins to output except GPIO0
	WPU    = 0b00000000;	// Set weak pull-up off on all pins

	// Set up the Analogue to Digital convertors (AN0 is used for
	// determining the button states) - since we have 4Mhz clock
	// we will use a 8 Tosc conversion time (2uS)
	ADCON0 = 0b10000001; // Right justified, Vdd as reference, AN0
	ANSEL  = 0b00010001; // Fosc/8, AN0 selected
		
	// Set up the comparators
	CMCON0 = 7;				// Disable comparator 0
	CMCON1 = 0;				// Disable comparator 1		
	
	// Reset button can bounce a little, so here we have a short
	// power delay to prevent the game start from 'stuttering'
	__delay_ms(200);
	
	// The actual game code is a polled finite state machine
	// which we have to call every 10 milliseconds
	// 
	// Note: we don't use an interrupt for this since it would
	// shift the timing of the sound generation causing it to
	// warble.

	// Main loop
	while(1)
	{
		// Set the game over flag
		unsigned char gameOver = FALSE;
		
		// Set up timer0 to measure the poll timing
		
		// Since the clock is 4Mhz the Fosc/4 is 1,000,000 timer ticks per second,
		// so the required timer value is 10,000 ticks (10 milliseconds)
		// By using a prescaler of 1:64 we can reduce the figure to
		// 156 ticks which is in range of the 8-bit counter
		// 255 - 156 = 99.
		OPTION = 0b00000101;	// Set timer0 prescaler to 1:64
		T0IF = 0;	// Clear timer0 interrupt flag
		T0IE = 0; // Disable timer0 interrupts
		TMR0 = 99;	// Set timer0's counter value

		// Get the required skill level
		skillLevel = 0;
		unsigned int timerCounter = 0;
		unsigned char ledFlash = 1;
		
		// Note: Random number generation is based on a repeating loop from
		// 1 to 4.  Since the game actions are based on when the user presses
		// a button (which is effectively a random event in time) the number
		// (when read by the game code) will be random.
		
		while (skillLevel == 0)
		{
			// Increment the random number
			randomNumber++;
			if (randomNumber == 5) randomNumber = 1;
			
			// Read the button states
			readButtonState();
			
			if (timerCounter == 0)
			{
				led(ledFlash);
				ledFlash++;
				if (ledFlash == 5) ledFlash = 1;
				timerCounter++;
			}
			
			if (T0IF)
			{
				timerCounter++;
				if (timerCounter == 20) timerCounter = 0;
				
				// Reset timer0
				TMR0 = 99;
				T0IF = 0;
			}
			
			// Is a button pressed?
			if (currentButtonState[BLUE] == ON) skillLevel = 1;
			if (currentButtonState[RED] == ON) skillLevel = 2;
			if (currentButtonState[GREEN] == ON) skillLevel = 3;
			if (currentButtonState[WHITE] == ON) skillLevel = 4;	
		}
		
		// Turn off the LEDs
		led(OFF);
		__delay_ms(400);

		// Play the game
		while (!gameOver)
		{
			// Increment the random number
			randomNumber++;
			if (randomNumber == 5) randomNumber = 1;
			
			// Read the button states
			readButtonState();
			
			// Timer0 overflow?
			if (T0IF)
			{
				// Poll the game's state machine
				gameOver = pollGameStateMachine();
					
				// Reset timer0
				TMR0 = 99;
				T0IF = 0;
			}		
		} // Game over
		
		// Go to sleep
		goToSleep();
	}		
}

#endif