/************************************************************************
	simon.c

    MicroSimon (MB Electronics Simon Clone)
    Copyright (C) 2011 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef SIMON_C
#define SIMON_C

// Global includes
#include <htc.h>

// Local includes
#include "hardware.h"
#include "simon.h"
#include "sound.h"
#include "charlieplex.h"
#include "buttons.h"

// Poll the game state machine.  Returns FALSE if game is in progress
// and TRUE is game is over
unsigned char pollGameStateMachine(void)
{
	unsigned char gameOver = FALSE;
	
	switch (gameCurrentState)
	{
		// State: Playing sequence - colour
		case GS_PLAYING_SEQUENCE_COLOUR:
			// If we are at the end of the current sequence add another colour
			if (sequenceNumber == coloursInSequence)
			{
				// Gat a random number (1-4)
				gameSequence[sequenceNumber] = randomNumber;
			}
			
			// Turn on the light and sound
			switch (gameSequence[sequenceNumber])
			{
				case BLUE:
					led(BLUE);
					soundOn(S_BLUE);
					break;
					
				case RED:
					led(RED);
					soundOn(S_RED);
					break;
				
				case GREEN:
					led(GREEN);
					soundOn(S_GREEN);
					break;
					
				case WHITE:
					led(WHITE);
					soundOn(S_WHITE);
					break;
			}
			
			// Reset the timer
			gameTimer = 0;
			
			// Switch state to hold colour
			gameCurrentState = GS_PLAYING_SEQUENCE_HOLD_COLOUR;
			
			break; // End of GS_PLAYING_SEQUENCE_COLOUR
			
		// State: Playing sequence - hold colour
		case GS_PLAYING_SEQUENCE_HOLD_COLOUR:
			gameTimer++;
			
			// Pick the required hold time based on the current sequence position
			unsigned char holdTime = 0;
			
			if (coloursInSequence < 5) holdTime = 42;
			if (coloursInSequence >= 5) holdTime = 32;
			if (coloursInSequence >= 13) holdTime = 22;
			
			// Hold the colour for the correct amount of time
			if (gameTimer >= holdTime)
			{
				// Turn off the light and sound
				led(OFF);
				soundOff();
				
				// Change state
				gameTimer = 0;
				gameCurrentState = GS_PLAYING_SEQUENCE_PAUSE;
			}	
			
			break; // End of GS_PLAYING_SEQUENCE_HOLD_COLOUR
			
		// State: Playing sequence - pause
		case GS_PLAYING_SEQUENCE_PAUSE:
			// Have we finished playing back the sequence?
			if (sequenceNumber == coloursInSequence)
			{
				// Finished playing the sequence, now it's time for the user to play it back
				sequenceNumber = 0;
				gameTimer = 0;
				gameCurrentState = GS_WAITING_FOR_COLOUR;
			}
			else
			{
				gameTimer++;
				
				// Wait for 0.05 seconds
				if (gameTimer >= 5)
				{
					// Play the next colour in the sequence
					gameCurrentState = GS_PLAYING_SEQUENCE_COLOUR;
					sequenceNumber++;
				}
			}		
			break; // End of GS_PLAYING_SEQUENCE_PAUSE
			
		// State: Waiting for a colour to be pressed
		case GS_WAITING_FOR_COLOUR:
			// Is a colour button being pressed?
			if (currentButtonState[BLUE] == ON ||
				currentButtonState[RED] == ON ||
				currentButtonState[GREEN] == ON || 
				currentButtonState[WHITE] == ON)
			{
				// Colour button pressed - is it the right colour?
				unsigned char correctColour = FALSE;
				if (gameSequence[sequenceNumber] == BLUE &&
					currentButtonState[BLUE] == ON) correctColour = TRUE;
				if (gameSequence[sequenceNumber] == RED &&
					currentButtonState[RED] == ON) correctColour = TRUE;
				if (gameSequence[sequenceNumber] == GREEN &&
					currentButtonState[GREEN] == ON) correctColour = TRUE;
				if (gameSequence[sequenceNumber] == WHITE &&
					currentButtonState[WHITE] == ON) correctColour = TRUE;
					
				if (correctColour == TRUE)
				{
					// Light the pressed colour and play the sound
					if (currentButtonState[BLUE] == ON)
					{
						led(BLUE);
						soundOn(S_BLUE);
					}
					
					if (currentButtonState[RED] == ON)
					{
						led(RED);
						soundOn(S_RED);
					}
					
					if (currentButtonState[GREEN] == ON)
					{
						led(GREEN);
						soundOn(S_GREEN);
					}
					
					if (currentButtonState[WHITE] == ON)
					{
						led(WHITE);
						soundOn(S_WHITE);
					}
					
					// Wait for the button to be released
					gameCurrentState = GS_WAITING_FOR_RELEASE;
				}
				else
				{
					// Play the losing tone	
					soundOn(S_FAIL);
					
					// Light the incorrect colour which was pressed
					if (currentButtonState[BLUE] == ON) led(BLUE);	
					if (currentButtonState[RED] == ON) led(RED);	
					if (currentButtonState[GREEN] == ON) led(GREEN);			
					if (currentButtonState[WHITE] == ON) led(WHITE);
					
					gameTimer = 0;
					gameCurrentState = GS_GAME_OVER_LOSE;
				}
				break;
				
			}
			else
			{
				// No colour button is being pressed
				gameTimer++;
				
				// Have we timed out waiting for a colour?
				if (gameTimer >= 300)
				{
					// Play the loosing tone
					soundOn(S_FAIL);
					
					// Light the expected colour
					if (gameSequence[sequenceNumber] == BLUE) led(BLUE);
					if (gameSequence[sequenceNumber] == RED) led(RED);
					if (gameSequence[sequenceNumber] == GREEN) led(GREEN);
					if (gameSequence[sequenceNumber] == WHITE) led(WHITE);
					
					// Switch states
					gameTimer = 0;
					gameCurrentState = GS_GAME_OVER_LOSE;
				}	
			}	
			break; // End of GS_WAITING_FOR_COLOUR
		
		// State: Game over lose	
		case GS_GAME_OVER_LOSE:
			gameTimer++;
			
			// Play the losing tone for 1.5 seconds
			if (gameTimer >= 150)
			{
				// Turn off the light and sound
				led(OFF);
				soundOff();
		
				// Game over
				gameOver = TRUE;
			}	
			break; // End of GS_GAME_OVER_LOSE
			
		// State: Waiting for button release
		case GS_WAITING_FOR_RELEASE:
			// Is a colour button being pressed?
			if (currentButtonState[BLUE] == ON ||
				currentButtonState[RED] == ON ||
				currentButtonState[GREEN] == ON || 
				currentButtonState[WHITE] == ON)
			{
				// Colour button is being pressed
			}	
			else
			{
				// No colour button is being pressed
				
				// Turn off the light and sound
				led(OFF);
				soundOff();
				
				// Was that the last colour in the sequence?
				if (coloursInSequence == sequenceNumber)
				{
					// Select the required winning sequence length based on the skill level
					unsigned char sequenceLengthToWin = 0;
					if (skillLevel == 1) sequenceLengthToWin = 7;
					if (skillLevel == 2) sequenceLengthToWin = 13;
					if (skillLevel == 3) sequenceLengthToWin = 19;
					if (skillLevel == 4) sequenceLengthToWin = 30;
					
					if (coloursInSequence < sequenceLengthToWin)
					{
						// Game is not won, pause and then replay the sequence
						coloursInSequence++;
						sequenceNumber = 0;
						gameTimer = 0;
						
						gameCurrentState = GS_PAUSE_BEFORE_REPLAY;
					}
					else
					{
						// Wait for the next colour in the sequence
						gameTimer = 0;
						gameCurrentState = GS_GAME_OVER_WIN;
					}
				}
				else
				{
					// Get the next colour in the sequence
					sequenceNumber++;
					gameTimer = 0;
					gameCurrentState =	GS_WAITING_FOR_COLOUR;
				}
			}		
			break; // End of GS_WAITING_FOR_RELEASE
			
			// State: Pause before replaying colour sequence
			case GS_PAUSE_BEFORE_REPLAY:
				gameTimer++;
				
				// Wait for 0.8 seconds
				if (gameTimer >= 80)
				{
					gameTimer = 0;
					gameCurrentState = GS_PLAYING_SEQUENCE_COLOUR;
				}	
				break; // End of PAUSE_BEFORE_REPLAY
				
			// State: Game over - player wins
			case GS_GAME_OVER_WIN:
				if (gameTimer == 0+8 ||
					gameTimer == 5+8 ||
					gameTimer == 16+8 ||
					gameTimer == 26+8 ||
					gameTimer == 36+8 ||
					gameTimer == 46+8)
				{
					// Show the winning colour
					if (gameSequence[sequenceNumber] == BLUE) led(BLUE);
					if (gameSequence[sequenceNumber] == RED) led(RED);
					if (gameSequence[sequenceNumber] == GREEN) led(GREEN);
					if (gameSequence[sequenceNumber] == WHITE) led(WHITE);
					
					// Play the winning sound
					if (gameSequence[sequenceNumber] == BLUE) soundOn(S_BLUE);
					if (gameSequence[sequenceNumber] == RED) soundOn(S_RED);
					if (gameSequence[sequenceNumber] == GREEN) soundOn(S_GREEN);
					if (gameSequence[sequenceNumber] == WHITE) soundOn(S_WHITE);
				}
				
				// Winner sequence finished?
				if (gameTimer == 2+8 ||
					gameTimer == 13+8 ||
					gameTimer == 23+8 ||
					gameTimer == 33+8 ||
					gameTimer == 43+8 ||
					gameTimer == 53+8)
				{
					// Turn off the lights and sound
					led(OFF);
					soundOff();
					
					// Game over?
					if (gameTimer == 53+8) gameOver = TRUE;
				}
				
				gameTimer++;
				
				break; // End of GS_GAME_OVER_WIN
	} // End of switch (gameCurrentState)
	
	return gameOver;
}	

#endif