/************************************************************************
	sound.c

    MicroSimon (MB Electronics Simon Clone)
    Copyright (C) 2011 Simon Inns

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Email: simon.inns@gmail.com

************************************************************************/

#ifndef SOUND_C
#define SOUND_C

// Global includes
#include <htc.h>

// Local includes
#include "hardware.h"
#include "sound.h"

// Sound generation interrupt handler
void interrupt isr()
{
	// Is this timer1 interrupting? (PWM sound generation interrupt)
	if (TMR1IF)
	{
		// Pulse on or pulse off?
		if (soundPwmState == OFF)
		{
			SPEAKER = 1;
			soundPwmState = ON;
		}
		else
		{
			SPEAKER = 0;
			soundPwmState = OFF;
		}

		// Reset the timer1 counter bytes
		TMR1H = soundTimer1HighByte;
		TMR1L = soundTimer1LowByte;

		// Reset the timer1 interrupt flag
		TMR1IF = 0;
	}
}

// Turn sound on
void soundOn(int frequency)
{
	// Check that we are not already making a sound
	if (soundState == ON) return;
	else soundState = ON;

	int timerValue;

	// We will use timer1 to generate the PWM by turning the pulse
	// on and off at a 50% duty cycle

	// Timer1 must be called at twice the desired frequency in order to
	// turn the pulse on and off (at 50% duty cycle).  Since the clock
	// is 4Mhz the Fosc/4 is 1,000,000 timer ticks per second, so the required
	// timer value is 500,000 ticks / frequency

	// By using a prescaler of 1:4 we can reduce the figure to
	// 125,000 ticks / frequency

	// Since the timer is 16-bit the maximum timer setting is 65535, meaning that
	// the lowest frequency we can generate is around 23 Hz
	// however we limit the range to 30-20,000 Hz to be on the safe-side

	// Sanity check the frequency
	if (frequency < 30) frequency = 30;
	if (frequency > 20000) frequency = 200000;

	timerValue = 65536 - (125000 / frequency);

	// Set the TMR1L and TMR1H bytes according to timerValue
	soundTimer1HighByte = (char)(timerValue >> 8);
	soundTimer1LowByte =  (char)timerValue;

	TMR1H = soundTimer1HighByte;
	TMR1L = soundTimer1LowByte;

	// Set the PWM sound state to ON (used by the interrupt to track if the pulse
	// should be high or low)
	soundPwmState = ON;

	// Enable the interrupts
	PEIE = 1;
	GIE = 1;

	// Set up the timer
	TMR1IF = 0; // Reset the timer1 interrupt flag
	T1SYNC = 0;
	T1CON = 0b00100001;
	TMR1IE = 1; // Turn on the timer1 interrupt

}

// Turn sound off
void soundOff(void)
{
	// This routine stops the sound by disabling the timer1 interrupt
	TMR1IE = 0;
	T1CON = 0b00100000;

	// Set the sound state to OFF
	soundState = OFF;

	// Make sure the port is low
	SPEAKER = OFF;
}

#endif